{
  TFileList - a File ListView component for Delphi 2
   Amedeo Lanza <amldc@tin.it>

description:
 TFileList is a TCustomListView descendant wich encapsulates some system
 functions to behave somehow like an Explorer's file window. May show file
 informations ignored by Explorer.

copyright & license
 This source is Copyright  of Amedeo Lanza di Casalanza. You may freely use,
 modify and distribute this software for non commercial purposes and for writing
 freeware and/or shareware software. You MAY NOT SELL this software although you
 may include it in software collection you distribute, provided there is no
 charge for the software itself. Please leave the copyright information and any
 additional information (readme.* ecc.) provided with the original author's
 copy.
 Use of this software for building of commercial programs should be expressely
 authorized by the author.
 Any use of this software for illegal activities is prohibited.

 If you enhance this software or fix any bug, please send the new source to
 the author.

DISCLAIMER:
 I ASK NO FEE and I GIVE NO WARRANTY for this software, either expressed or
 implied. Use it AT YOUR OWN RISK.

suggestions, bug reports & comments to the author:
 Amedeo Lanza di Casalanza
 from: Torino, Italy
 mailto:amldc@tin.it
 http://volftp.tin.it/IT/IT/COLLABORAZIONI/LANZA/index.htm

WARNING FOR EMAIL ABUSE:
 *BORLAND DELPHI* related messages are WELCOME, any other use of the author's
 email address for unsolicited ADVERTISING is STRICTLY PROHIBITED.

aknowledgement:
 Portions of this software come from freeware examples by
  Markus Stephany
  MirBir.St@T-Online.de
  http://home.t-online.de/home/MirBir.St/
 wich I found on Delphi Super Page (http://SunSITE.icm.edu.pl/delphi/)

also thanks for hints to the very good "Unofficial Newsletter of Delphi Users"
(UNDU) at http://www.informant.com/undu/index.htm

and finally ...the TFileList class !

 properties:

  Directory: string
   Setting this property will cause TFileList to scan the specified path for
   files and load its items with files informations

  DisplayDirectory: boolean
   If set to TRUE will enable the loading of directories' info, when TRUE the
   directories will be ignored.

  Mask: string;
   File specifications list as in a TFilterComboBox.Mask
   May be single file spec or list of filespecs separated by semicolon or
   commas (always returns with semicolons).

  SortColumn :TFileInfo
   Column to be used for sorting

   TFileInfo is defined as
   . fiName       file name
   . fiSize       file size
   . fiType       file type as registered by windows
   . fiModified   date of last write
   . fiCreated    date of creation
   . fiLastAccess date of last access
   . fiAttributes attributes
   . fiDosName    alternate dos name (8.3)
   . fiPath       file's directory
   and is used to select the desired items

  ViewColumns :TFileInfos
   Set of TFileInfo used to select the desired columns to display. Each column
   is associated to a file information.

 all properties published by TListView are also published, execpt
  Columns     - handled by internal routines
  LargeImages - load system image list in the create method
  SmallImages - load system image list in the create method
  StateImages - not used

 public methods:
  procedure AddMask (Mask :string)
  Add specified file mask to current file mask. May add single file spec or
  list of filespecs separated by semicolon or commas.

  procedure SetColCaption (Col: TFileInfo; Value :string);
  set specified column's title to Value

  procedure SetColWidth   (Col: TFileInfo; Value :integer);
  set the specified column's width to value

 events:
  ColClick (from TListView) is overridden in order to call a sort ruotine
  before the inherithed method

comments:
  uses FindFirstFile, FindNextFile and FindClose API calls from Windows unit to
  retrieve file list with better performances and ShGetFileInfo to retrieve
  extended informations.
  To find help about TWin32FindData look for WIN32_FIND_DATA in the Win32
  Programmer's Reference and TWin32FindData in Windows unit (if you have the
  RTL sources you can find it in the source\rtl\win directory.

current known limitations :
  Tested only with Delphi 2.0 and Win95, requires anyway Win32s,Win95 or WinNT
  as uses Win32 API calls.
  Handles only File Items, System Resources cannot be displayed as in an
  Explorer's window.
  Columns order is fixed, and settings of columns' width and caption is quite
  trivial; I hope I (or someone else) will be able to add a property editor for
  that.

history:
 10-aug-1997 - V1.0
  amldc@tin.it - first development

}

unit FileList;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls,ShellApi;

type

  TFileInfo = (fiName,fiSize,fiType,
               fiModified,fiCreated,fiLastAccess,
               fiAttributes,fiDosName,fiPath);
  TFileInfos = Set of TFileInfo;

  TColumnInfo = record
    Caption   :TCaption;
    Width     :integer;
    Alignment :TAlignment;
    Column    :integer;
  end;
  TColumnInfos = array [TFileInfo] of TColumnInfo;
  TInfoColumn = array [0..Ord(Pred(fiPath))] of TFileInfo;

  TFileList = class(TCustomListView)
  private
    { Private declarations }
    fDirectory    :string;        // current directory
    fMask    :TStringList;        // file specifications
    fDisplayDir   :boolean;       // include directories in file scan
    fViewColumns  :TFileInfos;    // visible columns
    fSortColumn   :TFileInfo;     // current sort column
    procedure GetSystemImageList; // load LargeImages and SmallImages
    procedure UpdateFileList;     // scan the directory and load file list
    procedure SetupFileColumnInfos;   // dynamic creation of columns
  protected
    { Protected declarations }
    procedure ColClick(Column: TListColumn); override;
    procedure SetDirectory   (Directory :string); // update fDirectory and update
    procedure SetDisplayDir  (Display :boolean);  // enable/disable list of dirs
    function  GetMask :string;               // Get file specifications
    procedure SetMask   (Mask :string); // Set file specifications
    procedure SetSortColumn  (Column :TFileInfo);
    procedure SetViewColumns (Columns :TFileInfos);
    procedure Loaded; override;
    procedure SetupFileColumns;
  public
    { Public declarations }
    constructor Create (aOwner :TComponent); override;
    destructor Destroy; override;
    procedure AddMask   (Mask :string); // Add file specifications
    procedure AddFile   (Path :string; FindData :TWin32FindData);
    procedure SetColCaption (Col: TFileInfo; Value :string);
    procedure SetColWidth   (Col: TFileInfo; Value :integer);
  published
    { Published declarations }
    property Align;
    property BorderStyle;
    property Color;
    property ColumnClick;
    property OnClick;
    property OnDblClick;
    property Ctl3D;
    property DragMode;
    property ReadOnly;
    property Font;
    property HideSelection;
    property IconOptions;
    property Items;
    property AllocBy;
    property MultiSelect;
    property OnChange;
    property OnChanging;
    property OnColumnClick;
    property OnCompare;
    property OnDeletion;
    property OnEdited;
    property OnEditing;
    property OnEnter;
    property OnExit;
    property OnInsert;
    property OnDragDrop;
    property OnDragOver;
    property DragCursor;
    property OnStartDrag;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property ParentShowHint;
    property ShowHint;
    property PopupMenu;
    property ShowColumnHeaders;
    property SortType;
    property TabOrder;
    property TabStop default True;
    property ViewStyle;
    property Visible;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property Directory :string read fDirectory write SetDirectory;
    property DisplayDirectory :boolean read fDisplayDir write SetDisplayDir default true;
    property ViewColumns :TFileInfos read fViewColumns write SetViewColumns;
    property SortColumn :TFileInfo read fSortColumn write SetSortColumn default fiName;
    property Mask :string read GetMask write SetMask;
  end;

procedure Register;

implementation

var
  InvertSort :boolean; // used to invert sorting when column clicked twice
  fColumnInfos  :TColumnInfos;  // columns informations
  fInfoColumn   :TInfoColumn;   // columns informations

//
// UTILITIES
//

//
// Sorting Routine
//
function SortProc (Item1,Item2 :TListItem; ColIndex :integer) :integer; stdcall;
var
  Caption1,Caption2,
  String1,String2,
  Attributes1,Attributes2 :string;
begin
  Result := 0; // Defaults to equal
  // Tests Column[1] (Attributes) to force Directories before Files
  if (Item1.SubItems[0] = '') and (Item2.SubItems[0] <> '') then Result := -1
  else
    // Tests Column[1] (Attributes) to force Directories before Files
    if (Item1.SubItems[0] <> '') and (Item2.SubItems[0] = '') then Result := 1
    else
    //
    // both items are directory or file
    //
    begin
      // if ColIndex is 0, just a sort by NAME is required
      if ColIndex < 1 then
      begin
        // Converts NAME to uppercase to ignore case
        Caption1 := AnsiUpperCase(Item1.Caption);
        Caption2 := AnsiUpperCase(Item2.Caption);
        // Compare NAMES
        if Caption1 > Caption2 then Result := 1 else
        if Caption1 < Caption2 then Result := -1;
      end
      else
      begin
        // checks for invalid column specified (1st item)
        if Item1.SubItems.Count < ColIndex then String1 := ''
        else String1 := AnsiUpperCase(Item1.SubItems[ColIndex-1]);
        // checks for invalid column specified (2nd item)
        if Item2.SubItems.Count < ColIndex then String2 := ''
        else String2 := AnsiUpperCase(Item2.SubItems[ColIndex-1]);

        // compare the selected values
        if String1 > String2 then Result := 1 else
          if String1 < String2 then Result := -1 else
          {if String1 = String2 then} // stings are equal, try to sort on Caption
            begin
              // Converts NAME to uppercase to ignore case
              Caption1 := AnsiUpperCase(Item1.Caption);
              Caption2 := AnsiUpperCase(Item2.Caption);
              // Compare NAMES
              if Caption1 > Caption2 then Result := 1 else
              if Caption1 < Caption2 then Result := -1
            end;
      end;
    // invert Sort if requested
    if InvertSort then Result := Result * -1; // is sort reverted ?
    // Date-Time field sorted in reverse order
    if fColumnInfos[fiCreated].Column = ColIndex then Result := Result * -1
    else if fColumnInfos[fiModified].Column = ColIndex then Result := Result * -1
    else if fColumnInfos[fiLastAccess].Column = ColIndex then Result := Result * -1;
  end;
end;

//
// STRING ROUTINES
//


// Get Shell Info for the specified file
procedure GetShellFileInfo (FileName :TFileName; var ShFileInfo :TShFileInfo);
begin
  ShGetFileInfo (PChar(FileName),0,ShFileInfo,SizeOf (ShFileInfo),
                 shgfi_SysIconIndex or shgfi_Icon or
                 shgfi_DisplayName or shgfi_TypeName or
                 shgfi_SmallIcon);
end;

// Convert Size for Sort
function SizeStr (Size,Typ :integer ) :string;
begin
  if (Typ and faDirectory) = faDirectory then
    Result := ''
  else
    Result := Format ('%10d',[Size]);
end;

// returns a string with file attributes (DRSH)
function AttrStr(Attr:integer):string;
begin
  Result := '';
  if (Attr and file_attribute_Directory)  > 0 then Result := Result + 'D';
  if (Attr and file_attribute_Archive)    > 0 then Result := Result + 'A';
  if (Attr and file_attribute_Readonly)   > 0 then Result := Result + 'R';
  if (Attr and file_attribute_System)     > 0 then Result := Result + 'S';
  if (Attr and file_attribute_Hidden)     > 0 then Result := Result + 'H';
//  if (Attr and FILE_ATTRIBUTE_COMPRESSED) > 0 then Result := Result + 'C';
  if (Attr and file_attribute_Temporary)  > 0 then Result := Result + 'T';
end;

// File Date & Time
function GetTimeModified(a:tfiletime):string;
// This function retrieves the last time, the given file was written to disk
var
  mtm :TSystemTime;
  at  :TFileTime;
  ds,ts:ShortString;
begin
  // Time must get converted, else there is an error of one hour
  // Does anybody know what this function does ?
  // Maybe something like summertime/wintertime (or what you call it out of Germany) ?
  filetimetolocalfiletime(a,at);
  filetimetosystemtime(at,mtm);
  SetLength(ds, GetDateFormat(LOCALE_USER_DEFAULT, 0, @mtm, NIL, @ds[1], 255) - 1);
  SetLength(ts, GetTimeFormat(LOCALE_USER_DEFAULT, time_noseconds, @mtm, NIL,
                                               @ts[1], 255)  - 1);
  Result:=ds+'  '+ts;
end; // End getmod

//
// CLASS METHODS FOR TFileList
//

// Object creation
constructor TFileList.Create (aOwner :TComponent);
begin
  inherited Create(aOwner);
  fDisplayDir := true;   // include directory in display
  fSortColumn := fiName; // set default sort to Name
  fViewColumns := [fiName,fiSize,fiType,fiModified,fiCreated,
                   fiLastAccess,fiDosName,fiAttributes];
  GetSystemImageList;    // get system icon list
  SetupFileColumnInfos;  // load column configuration
  InvertSort := false;   // default is normal sort
  fMask := TStringList.Create;   // File Specifications list
  fMask.Sorted := true;          // sorted and
  fMask.Duplicates := dupIgnore; // ignore duplicates
  fMask.Add('*.*');
  //Mask := '*.*';
end;

procedure TFileList.Loaded;
begin
  inherited Loaded;
  SetupFileColumns;      // build listview columns
end;

// Object deletion
destructor TFileList.Destroy;
begin
  fMask.Free;        // releases child component
  inherited Destroy;
end;
//
// INITIALIZATION
//
// Get System Image List
procedure TFileList.GetSystemImageList;
var
  SysImageList :uint;        // temporary handle for System ImageLists
  ShFileInfo :TShFileInfo; // Shell File Info structure
begin
  LargeImages := TImageList.Create(self);
  SysImageList := ShGetFileInfo('',0,ShFileInfo,SizeOf(ShFileInfo),shgfi_SysIconIndex or shgfi_LargeIcon);
  if SysImageList <> 0 then
  begin
    LargeImages.Handle := SysImageList;
    LargeImages.ShareImages := true; // Avoid freeing of System Image List !
  end;
  SmallImages := TImageList.Create(self);
  SysImageList := ShGetFileInfo('',0,ShFileInfo,SizeOf(ShFileInfo),shgfi_SysIconIndex or shgfi_SmallIcon);
  if SysImageList <> 0 then
  begin
    SmallImages.Handle := SysImageList;
    SmallImages.ShareImages := true; // Avoid freeing of System Image List !
  end;
end;

procedure TFileList.SetColCaption (Col: TFileInfo; Value :string);
begin
  if fColumnInfos[Col].Caption = Value then Exit;
  fColumnInfos[Col].Caption := Value;
end;
procedure TFileList.SetColWidth (Col: TFileInfo; Value :integer);
begin
  if fColumnInfos[Col].Width = Value then Exit;
  fColumnInfos[Col].Width := Value;
end;

// Sets up columns infos for ListView
procedure TFileList.SetupFileColumnInfos;
begin
  // column properties for NAME
  fColumnInfos[fiName].Caption := 'Name';
  fColumnInfos[fiName].Width := 150;
  fColumnInfos[fiName].Alignment := taLeftJustify;
  // column properties for SIZE
  fColumnInfos[fiSize].Caption := 'Size';
  fColumnInfos[fiSize].Width := 60;
  fColumnInfos[fiSize].Alignment := taRightJustify;
  // column properties for Type
  fColumnInfos[fiType].Caption := 'Type';
  fColumnInfos[fiType].Width := 130;
  fColumnInfos[fiType].Alignment := taLeftJustify;
  // column properties for Modified
  fColumnInfos[fiModified].Caption := 'Modified';
  fColumnInfos[fiModified].Width := 100;
  fColumnInfos[fiModified].Alignment := taLeftJustify;
  // column properties for Created
  fColumnInfos[fiCreated].Caption := 'Created';
  fColumnInfos[fiCreated].Width := 100;
  fColumnInfos[fiCreated].Alignment := taLeftJustify;
  // column properties for Last Access
  fColumnInfos[fiLastAccess].Caption := 'Last Access';
  fColumnInfos[fiLastAccess].Width := 85;
  fColumnInfos[fiLastAccess].Alignment := taLeftJustify;
  // column properties for Attributes
  fColumnInfos[fiAttributes].Caption := 'Attributes';
  fColumnInfos[fiAttributes].Width := 60;
  fColumnInfos[fiAttributes].Alignment := taLeftJustify;
  // column properties for DosName
  fColumnInfos[fiDosName].Caption := 'DosName';
  fColumnInfos[fiDosName].Width := 130;
  fColumnInfos[fiDosName].Alignment := taLeftJustify;
  // column properties for Path
  fColumnInfos[fiPath].Caption := 'Path';
  fColumnInfos[fiPath].Width := 200;
  fColumnInfos[fiPath].Alignment := taLeftJustify;
end;
// Builds columns for ListView
// NAME and SIZE column always created and just hidden if not
// required. Other columns are created only if requested
procedure TFileList.SetupFileColumns;
var
  Ind :integer;
  Inf :TFileInfo;
begin
  with Columns do
  begin
    Clear;
    // NAME
    with Add do // adds and sets up NAME column
    begin
      Caption   := fColumnInfos[fiName].Caption;
      Alignment := fColumnInfos[fiName].Alignment;
      fColumnInfos[fiName].Column    := 0; // set column index;
      if fiName in fViewColumns then // Hide column if not required
        Width := fColumnInfos[fiName].Width
      else
        Width := 0;
      fInfoColumn[0] := fiName;
    end;
    with Add do // adds and sets up SIZE column
    begin
      Caption   := fColumnInfos[fiSize].Caption;
      Alignment := fColumnInfos[fiSize].Alignment;
      fColumnInfos[fiSize].Column    := 1; // set column index;
      if fiSize in fViewColumns then // Hide column if not required
        Width := fColumnInfos[fiSize].Width
      else
        Width := 0;
      fInfoColumn[1] := fiSize;
    end;
    // all remaining columns
    Ind := 2; // start from column 2 (3rd column)
    for Inf := fiType  to fiPath do
      if Inf in fViewColumns then // Hide column if not required
      begin
        with Add do // adds and sets up SIZE column
        begin
          Caption   := fColumnInfos[Inf].Caption;
          Alignment := fColumnInfos[Inf].Alignment;
          Width     := fColumnInfos[Inf].Width;
          fColumnInfos[Inf].Column := Ind; // set column index;
          // set
          fInfoColumn[Ind] := Inf;
          Inc(Ind);
        end;
      end
      else
        fColumnInfos[Inf].Column := -1; // reset column index if column not used
  end; {with columns do}
end;


//
// EVENT HANDLERS
//

// overrides ancestor's handler for column headers click:
//  calls first the Set Sort Column procedure and then
//  the inherited ColClick method.
procedure TFileList.ColClick(Column: TListColumn);
begin
  SetSortColumn(fInfoColumn[Column.Index]);
  inherited ColClick(Column);
end;

//
// PROPERTIES INTERFACE
//

// Set File Specification
Procedure TFileList.SetMask (Mask :string);
begin
  fMask.Clear;
  AddMask(Mask);
end;
// Add specifications to File Specifications list
function TFileList.GetMask :string;
var
  I :integer;
begin
  Result := ''; // Default result to ''
  for I := 0 to Pred(fMask.Count) do
    Result := Result + fMask[I]  + ';';
  if Result[Length(Result)] = ';' then // remove last ';'
    Result := Copy(Result,1,Length(Result)-1);
end;

// Add specifications to File Specifications list
procedure TFileList.AddMask (Mask :string);
var
  Spec :string;
  I :integer;
begin
  Spec := '';
  I := 1;
  while I <= Length(Mask) do
  begin
    while (Mask[I] <> ';') and (Mask[I] <> ',') and (I <= Length(Mask))do
    begin // extracts characters for single file spec
      Spec := Spec + Mask[I];
      Inc(I); // next char
    end;
    if Spec <> '' then fMask.Add(Spec); //add File Specification to list
    Inc(I); // skip ';' or ','
    Spec := ''; // reset current spec
  end;
  UpdateFileList; // Update File List View
end;

// Set current column set
procedure TFileList.SetViewColumns (Columns :TFileInfos);
begin
  if Columns = fViewColumns then Exit;
  fViewColumns := Columns;
  if ViewStyle = vsReport then
  begin
    SetupFileColumns;      // rebuild listview columns
    UpdateFileList;
  end;
end;

// Set current sort column
procedure TFileList.SetSortColumn (Column :TFileInfo);
begin
  // invert sorting if the specified column was already selected
  if Column = SortColumn then InvertSort := not InvertSort
  else
    fSortColumn := Column;
  if ViewStyle = vsReport then
  begin
    CustomSort(@SortProc,fColumnInfos[SortColumn].Column); // Sorts on 0 based column index
  end;
end;

// Allows/Prevent loading of Directory Items in the file list
procedure TFileList.SetDisplayDir (Display :boolean);
begin
  if Display = fDisplayDir then Exit;
  fDisplayDir := Display;
  UpdateFileList;         // update file list
end;

// Set the directory for list view
// ensures the last character is a backslash
procedure TFileList.SetDirectory (Directory :string);
begin
  if Directory = fDirectory then Exit;
  fDirectory := Directory;
  if fDirectory[Length(fDirectory)] <> '\' then
    fDirectory := fDirectory + '\';
  if columns.count = 0 then
    SetupFileColumns;      // build listview columns
  UpdateFileList;          // update file listing
end;

//
// PROCESSING & SERVICES ROUTINES
//

// Add a file entry to the list view
procedure TFileList.AddFile (Path :string; FindData :TWin32FindData);
var
  ShFileInfo:TShFileInfo;
  NewItem :TListItem;
  function FileSize (hi,lo: integer) :longint;
  begin
    Result := (hi * MAXDWORD) + lo;
  end;
begin
  with FindData do
  begin
    NewItem := Items.Add;                           // Add Item to list
    NewItem.Caption := cFileName;                   // Set item caption to file NAME
    // Get Extended Info
    GetShellFileInfo (Path + cFileName,ShFileInfo); // Get Icon and Windows' Type
    NewItem.ImageIndex := ShFileInfo.IIcon;         // Set file icon index
    // SIZE
    NewItem.SubItems.Add (SizeStr(FileSize(nFileSizeHigh,nFileSizeLow),dwFileAttributes));
    // TYPE - from GetShellInfo extended info
    if fiType in fViewColumns then // add subitem only if requested
      NewItem.SubItems.Add (ShFileInfo.szTypeName); //type
    // MODIFIED
    if fiModified in fViewColumns then // add subitem only if requested
      NewItem.SubItems.Add (GetTimeModified(ftLastWriteTime));
    // CREATION
    if fiCreated in fViewColumns then // add subitem only if requested
      NewItem.SubItems.Add (GetTimeModified(ftCreationTime));
    // LAST ACCESS
    if fiLastAccess in fViewColumns then // add subitem only if requested
      NewItem.SubItems.Add (GetTimeModified(ftLastAccessTime));
    // ATTRIBUTES
    if fiAttributes in fViewColumns then // add subitem only if requested
      NewItem.SubItems.Add (AttrStr(dwFileAttributes));
    // DOS NAME
    if fiDosName in fViewColumns then // add subitem only if requested
      NewItem.SubItems.Add (cAlternateFileName);
   end;
  // PATH
  if fiPath in fViewColumns then // add subitem only if requested
    NewItem.SubItems.Add (Path);
end;

// Update File List with contents of the directory specified in
// fDirectory
procedure TFileList.UpdateFileList;
var
  FindData :TWin32FindData;
  FindHandle :THandle;
  Ret :boolean;
  OldViewStyle :TViewStyle;
  s :string;
  I :integer;
begin
  OldViewStyle := ViewStyle;
  Items.BeginUpdate;   // prevents video refresh until end of list loading
  Items.Clear;         // clear list view
  ViewStyle := vsList; // speeds up enumarating of contents
  Screen.Cursor := crHourGlass; // set cursor shape
  // First Scans for Subdirectories
    for I := 0 to Pred(fMask.Count) do
    begin
      FindHandle := Windows.FindFirstFile(PChar(fDirectory + fMask[I]),FindData);
      Ret := (FindHandle <> INVALID_HANDLE_VALUE);
      try
        while Ret do with FindData do
        begin
          if ((dwFileAttributes and file_attribute_Directory) = 0) then
            AddFile (fDirectory,FindData) // File entry - add to list
          else // adds directory entries only if required and not '.' or '..'
          begin
            S := FindData.cFileName;
            if fDisplayDir and (S <> '.') and (S <> '..') then
              AddFile (fDirectory,FindData);
          end;
          Ret := Windows.FindNextFile(FindHandle,FindData) // get next entry
        end;
      finally
        Windows.FindClose(FindHandle); // Close FindNext context
        // sort items by current sort item
        CustomSort(@SortProc,fColumnInfos[fSortColumn].Column);
        ViewStyle := OldViewStyle;
        Items.EndUpdate; // finally updates visual control
      end;
      Screen.Cursor := crDefault;
      end;
end;

// class registration
procedure Register;
begin
  RegisterComponents('Win95', [TFileList]);
end;

end.
